#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <string>
#include <vector>
// absolutní odchylka
static inline long double delta (const long p, const long q, const long double a) {
  return static_cast<long double>(p) / static_cast<long double>(q) - a;
}
// Výpočet nejlepší aproximace reálného desetinného čísla
// podílem 2 celých čísel pomocí řetězového zlomku
// r = aproximované číslo, n = počet iterací
#if 0
void retez (const long double r, const int n) {
  long pa = 0l, pb = 1l, qa = 1l, qb = 0l;      // vstupní hodnoty
  long double a = r;    // číslo shovej
  if (a < 0.0) {
    printf ("Zadane cislo musi byt kladne\n");
    return;
  }
  long double iz;
  for (int i=0; i<n; i++) {
    const long double z = modfl (a, &iz);        // rozděl na celou a desetinnou část
    const long k = static_cast<long int> (iz);   // celá část
    const long p = k * pb + pa;                  // p = dělenec
    const long q = k * qb + qa;                  // q = dělitel
    if ((p < 0) or (q < 0) or (z == 0.0)) break; // chyba
    const long double d = delta (p, q, r);
    printf ("%2d: %12ld / %-12ld, delta = %+Le\n", i, p, q, d);
    pa = pb; qa = qb; pb = p; qb = q;            // hodnoty do nového cyklu
    a = 1.0 / z;
  }
}
#else
static constexpr long TABMAX = 0x10000l;
static std::vector<long> primes;
static bool is_prime (const long n) {
  for (long i : primes) {
    ldiv_t d = ldiv (n, i);
    if (d.rem == 0l) return false;
    if (d.quot < i ) return true;
  }
  return false;
}
static void init_db () {
  printf("inicializuji tabulku prvocisel ... ");
  fflush(stdout);
  primes.push_back (2l);
  for (long n=3l; n<TABMAX; n+=2) {
    if (is_prime (n)) primes.push_back (n);
  }
  printf ("(%zd entries, %ld max) OK\n", primes.size(), primes [primes.size() - 1ul]);
}
static void rozklad_r (std::vector<long> & r, const long p) {
  bool b = true;
  for (long i : primes) {
    ldiv_t d = ldiv (p, i);
    if (d.rem == 0l) {
      r.push_back(i);
      return rozklad_r (r, d.quot);
    }
    if (d.quot < i) {
      b = false; break;
    }
  }
  if (b) printf ("%ld: rozklad nedokoncen - nutno zvetsit TABMAX !\n", p);
  if ((p == 1l) and (!r.empty())) return;
  r.push_back(p);
}
static const std::string rozklad (const long p) {
  if (primes.empty()) init_db ();
  std::vector<long> r;
  rozklad_r (r, p);
  //////// formatovani vystupu r //////////
  const int len = 0x400;
  char buf [len];
  int k = 0;
  buf [k++] = '(';
  for (long i : r) k += snprintf (buf + k, len - k,  "%ld*", i);
  k -= 1;
  buf [k++] = ')';
  buf [k++] = '\0';
  const std::string s (buf);
  return s;
}
static void frac_r (const long pa, const long pb, const long qa, const long qb,
                    const long double a, const long double r, const int n, const int m) {
  if (n == m) return;
  long double iz;
  const long double z = modfl (a, &iz);         // rozděl na celou a desetinnou část
  const long k = static_cast<long int> (iz);    // celá část
  const long p = k * pb + pa;                   // p = dělenec
  const long q = k * qb + qa;                   // q = dělitel
  if ((p < 0) or (q < 0) or (z == 0.0)) return; // chyba
//printf ("%2d: %12ld / %-12ld delta = %+Le\n", n + 1, p, q, delta (p, q, r));
  long double dd = delta (p, q, r);
  printf ("%2d: %20s / %-20s delta = %+Le\n", n + 1, rozklad(p).c_str(), rozklad(q).c_str(), dd);
  if (dd == 0.0) return;
  frac_r (pb, p, qb, q, 1.0 / z, r, n + 1, m);  // tail recursion
}
void retez (const long double r, const int n) {
  if (r < 0.0) {
    printf ("Zadane cislo musi byt kladne\n");
    return;
  }
  frac_r (0l, 1l, 1l, 0l, r, r, 0, n);
}
void test () {
  for (long n=0x240EDE20; n<0x240EDE40; n++) {
    if (is_prime (n)) printf ("%ld, %lX\n", n, n);
    //else printf("%s\n", rozklad(n).c_str());
  }
}
#endif
int main (int argc, char *argv[]) {
  long double pomer = M_PI;
  int cyklus = 15;
  switch (argc) {
    case 1:
      retez  (pomer, cyklus);
      printf ("Aproximace desetinneho cisla podilem celych cisel metodou retezoveho zlomku.\n");
      printf ("Pouziti:\n\t./retez desetinne.cislo [maximalni_pocet_aproximaci (default=15 max.)]\n");
      break;
    case 3:
      cyklus = strtol  (argv[2], nullptr, 10);
    case 2:
      pomer  = strtold (argv[1], nullptr);
      retez  (pomer, cyklus);
      break;
    default:
      printf ("Chybny pocet parametru\n");
      break;
  }
  return (0);
}

